

import java.awt.Point;
import java.io.PrintStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.PriorityQueue;
import java.util.Scanner;

/**
 * Solution to Step by Step (Easy)
 */
public class hopscotch_vanb
{
    
    /** The input. */
    private static Scanner sc;
    
    /** The output. */
    private static PrintStream ps;
    
    /**
     * A State of Dijkstra's algorithm.
     */
    private class State implements Comparable<State>
    {
        
        /** The current cell in the array. */
        public Point p;
        
        /** The distance so far. */
        public int d;        
        
        /**
         * Instantiates a new state.
         *
         * @param p the cell in the array
         * @param d the distance so far
         */
        public State( Point p, int d )
        {
            this.p = p;
            this.d = d;
        }

        /**
         * Compare this state to another by distance.
         *
         * @param other the other state
         * @return standard for compareTo
         */
        @Override
        public int compareTo( State other )
        {
            return d-other.d;
        }
    }
    
    /**
     * Do it.
     */
    private void doit()
    {
        int n = sc.nextInt();
        int k = sc.nextInt();
        
        List<Point>[] cells = new ArrayList[k+1];
        int array[][] = new int[n][n];
        boolean visited[][] = new boolean[n][n];
        
        for( int i=0; i<n; i++ ) Arrays.fill( visited[i], false );
        for( int i=0; i<k; i++ ) cells[i+1] = new ArrayList<Point>();
        
        for( int i=0; i<n; i++ ) for( int j=0; j<n; j++ )
        {
            array[i][j] = sc.nextInt();
            cells[array[i][j]].add( new Point(i,j) );
        }
        
        PriorityQueue<State> pq = new PriorityQueue<State>();
        for( Point start : cells[1] ) pq.add( new State( start, 0 ) );
        
        int answer = -1;
        
        while( !pq.isEmpty() )
        {
            State s = pq.poll();
            if( !visited[s.p.x][s.p.y] )
            {
                int i = array[s.p.x][s.p.y];
                if( i==k )
                {
                    answer = s.d;
                    break;
                }
                
                visited[s.p.x][s.p.y] = true;
                for( Point p : cells[i+1] ) if( !visited[p.x][p.y] )
                {
                      pq.add( new State( p, s.d + Math.abs( p.x-s.p.x ) + Math.abs( p.y-s.p.y ) ) );
                }
            }
        }
        
        ps.println( answer );
    }
    
    /**
     * The main method.
     *
     * @param args the arguments (unused)
     */
    public static void main( String[] args )
    {
        sc = new Scanner(System.in);
        ps = System.out;
        new hopscotch_vanb().doit();
    }

}
